#include <iostream>
#include <fstream>
#include <list>
#include <sys/time.h>
#include <unistd.h>

using namespace std;

const int MapX = 28;

class Map
{
public:
    enum CellType
    {
        CellField = ' ',
        CellBlock = '1',
        CellFrom = 'A',
        CellTo = 'B',
        CellPath = '*'
    };

public:
    void setCell(int x, int y, CellType cellType)
    {
        mMap[x][y] = cellType;
    }

    void calculatePath()
    {
        struct Cell { Cell(int x, int y, int s) : x(x), y(y), s(s) {} int x; int y; int s; };

        int map[MapX][10];
        for (int j = 0; j < 10; ++j)
            for (int i = 0; i < MapX; ++i)
                map[i][j] = -1;

        int startX = -1;
        int startY = -1;
        for (int j = 0; j < 10; ++j)
        {
            for (int i = 0; i < MapX; ++i)
            {
                if (mMap[i][j] == CellFrom)
                {
                    startX = i;
                    startY = j;
                    break;
                }
            }
            if (startX != -1 && startY != -1)
                break;
        }

        list<Cell> queue;
        queue.push_back(Cell(startX, startY, 0));
        map[startX][startY] = 0;
        while (!queue.empty())
        {
            Cell cell = queue.front();
            queue.pop_front();

            if (map[cell.x - 1][cell.y] == -1 && mMap[cell.x - 1][cell.y] != CellBlock)
            {
                map[cell.x - 1][cell.y] = cell.s + 1;
                queue.push_back(Cell(cell.x - 1, cell.y, cell.s + 1));
            }

            if (map[cell.x + 1][cell.y] == -1 && mMap[cell.x + 1][cell.y] != CellBlock)
            {
                map[cell.x + 1][cell.y] = cell.s + 1;
                queue.push_back(Cell(cell.x + 1, cell.y, cell.s + 1));
            }

            if (map[cell.x][cell.y - 1] == -1 && mMap[cell.x][cell.y - 1] != CellBlock)
            {
                map[cell.x ][cell.y - 1] = cell.s + 1;
                queue.push_back(Cell(cell.x, cell.y - 1, cell.s + 1));
            }

            if (map[cell.x][cell.y + 1] == -1 && mMap[cell.x][cell.y + 1] != CellBlock)
            {
                map[cell.x][cell.y + 1] = cell.s + 1;
                queue.push_back(Cell(cell.x, cell.y + 1, cell.s + 1));
            }
        }

        /*for (int j = 0; j < 10; ++j)
        {
            for (int i = 0; i < 10; ++i)
                printf("%02d", map[i][j]);
            cout << '\n';
        }*/

        int stopX = -1;
        int stopY = -1;
        for (int j = 0; j < 10; ++j)
        {
            for (int i = 0; i < MapX; ++i)
            {
                if (mMap[i][j] == CellTo)
                {
                    stopX = i;
                    stopY = j;
                    break;
                }
            }
            if (stopX != -1 && stopY != -1)
                break;
        }

        int currX = stopX;
        int currY = stopY;
        int step = map[currX][currY];

        while (step > 0)
        {
            step--;
            if (map[currX - 1][currY] == step)
            {
                mMap[currX - 1][currY] = CellPath;
                currX--;
                continue;
            }

            if (map[currX + 1][currY] == step)
            {
                mMap[currX + 1][currY] = CellPath;
                currX++;
                continue;
            }

            if (map[currX][currY-  1] == step)
            {
                mMap[currX][currY - 1] = CellPath;
                currY--;
                continue;
            }

            if (map[currX][currY + 1] == step)
            {
                mMap[currX][currY + 1] = CellPath;
                currY++;
                continue;
            }
        }
    }

    void debugPrint() const
    {
        for (int j = 0; j < 10; ++j)
        {
            for (int i = 0; i < MapX; ++i)
                cout << (char) mMap[i][j];
            cout << '\n';
        }
    }

private:
    CellType mMap[MapX][10];
};

double now()
{
    struct timeval start;

    long seconds, useconds;

    gettimeofday(&start, NULL);

    seconds  = start.tv_sec;
    useconds = start.tv_usec;

    return (((seconds) * 1000 + useconds/1000.0) + 0.5) * 0.001;
}

int main()
{
    Map map;

    ifstream mapFile;
    mapFile.open("map.txt");

    char buf[1024];
    for (int j = 0; j < 10; ++j)
    {
        mapFile.getline(buf, 1024);
        for (int i = 0; i < MapX; ++i)
        {
            map.setCell(i, j, Map::CellType(buf[i]));
        }
    }

    double a = now();

    for (int i = 0; i < 200 * 32; ++i)
        map.calculatePath();

    double b = now();
    std::cout << "sec: " << (b - a) << std::endl;

    map.debugPrint();
    return 0;
}

